@@ -1,5 +1,5 @@
 ;; maximize.jl -- window maximization
-;; $Id: maximize.jl,v 1.51 2000/11/03 22:11:45 jsh Exp $
+;; $Id: maximize.jl,v 1.55 2001/01/27 02:36:40 jsh Exp $
 
 ;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -27,6 +27,7 @@
 	    window-unmaximized-position
 	    window-unmaximized-dimensions
 	    window-maximizable-p
+	    frame-part-movable-p
 	    maximize-truncate-dims
 	    maximize-find-workarea
 	    maximize-window
@@ -86,6 +87,11 @@
     :user-level expert
     :type boolean)
 
+  (defcustom move-lock-when-maximized t
+    "Lock position and size while windows are maximized."
+    :type boolean
+    :group (min-max maximize))
+  
   ;; called when a window is maximized, args (W #!optional DIRECTION)
   (defvar window-maximized-hook nil)
 
@@ -275,21 +281,23 @@
 ;;; size hints stuff
 
   (define (window-maximizable-p w #!optional direction hints)
-    (unless hints
-      (setq hints (window-size-hints w)))
-    (let ((max-width (cdr (assq 'max-width hints)))
-	  (max-height (cdr (assq 'max-height hints)))
-	  (dims (window-dimensions w)))
-      (catch 'out
-	(when (and (or (null direction) (eq direction 'horizontal))
-		   max-width
-		   (>= (car dims) max-width))
-	  (throw 'out nil))
-	(when (and (or (null direction) (eq direction 'vertical))
-		   max-height
-		   (>= (cdr dims) max-height))
-	  (throw 'out nil))
-	t)))
+    (if (window-get w 'never-maximize)
+	nil
+      (unless hints
+	(setq hints (window-size-hints w)))
+      (let ((max-width (cdr (assq 'max-width hints)))
+	    (max-height (cdr (assq 'max-height hints)))
+	    (dims (window-dimensions w)))
+	(catch 'out
+	  (when (and (or (null direction) (eq direction 'horizontal))
+		     max-width
+		     (>= (car dims) max-width))
+	    (throw 'out nil))
+	  (when (and (or (null direction) (eq direction 'vertical))
+		     max-height
+		     (>= (cdr dims) max-height))
+	    (throw 'out nil))
+	  t))))
 
   (define (maximize-truncate-dims w dims #!optional direction hints)
     (unless hints
@@ -325,6 +333,18 @@ doesn't overlap any avoided windows, or nil."
 		   #:include-heads (list (or head (current-head))))))
       (find-max-rectangle avoided edges (current-head w))))
 
+  (define (frame-part-movable-p w part)
+    (if (not move-lock-when-maximized)
+	t
+      (let ((h-maximized (window-maximized-horizontally-p w))
+	    (v-maximized (window-maximized-vertically-p w)))
+	(case part
+	  ((top-border bottom-border) (not v-maximized))
+	  ((left-border right-border) (not h-maximized))
+	  ((top-left-corner top-right-corner
+	    bottom-left-corner bottom-right-corner title)
+	   (not (and h-maximized v-maximized)))))))
+
 
 ;;; commands
 
@@ -368,7 +388,7 @@ doesn't overlap any avoided windows, or nil."
 
   ;; does all unmaximizing except for changing the window properties and
   ;; calling the hooks
-  (define (unmaximize-window-1 w #!optional direction)
+  (define (unmaximize-window-1 w #!optional direction before)
     (let ((geom (window-get w 'unmaximized-geometry))
 	  (coords (window-position w))
 	  (dims (window-dimensions w)))
@@ -379,19 +399,22 @@ doesn't overlap any avoided windows, or nil."
 	(when (or (null direction) (eq direction 'vertical))
 	  (rplacd coords (nth 1 geom))
 	  (rplacd dims (nth 3 geom)))
+	(when before
+	  (before))
 	(move-resize-window-to w (car coords) (cdr coords)
 			       (car dims) (cdr dims)))))
 
   (define (unmaximize-window w #!optional direction)
     "Restore the dimensions of the window to its original, unmaximized, state."
-    (unmaximize-window-1 w direction)
-    (when (or (null direction) (eq direction 'horizontal))
-      (window-put w 'maximized-horizontally nil))
-    (when (or (null direction) (eq direction 'vertical))
-      (window-put w 'maximized-vertically nil))
-    (when (and (not (window-maximized-vertically-p w))
-	       (not (window-maximized-horizontally-p w)))
-      (window-put w 'unmaximized-geometry nil))
+    (unmaximize-window-1 w direction
+     (lambda ()
+       (when (or (null direction) (eq direction 'horizontal))
+	 (window-put w 'maximized-horizontally nil))
+       (when (or (null direction) (eq direction 'vertical))
+	 (window-put w 'maximized-vertically nil))
+       (when (and (not (window-maximized-vertically-p w))
+		  (not (window-maximized-horizontally-p w)))
+	 (window-put w 'unmaximized-geometry nil))))
     (call-window-hook 'window-unmaximized-hook w (list direction))
     (call-window-hook 'window-state-change-hook w (list '(maximized))))
 
@@ -510,6 +533,7 @@ unmaximized."
 	(window-put w 'maximize-removed-maximize-button nil))))
 
   (define (property-notify w prop type)
+    (declare (unused type))
     (when (eq prop 'WM_NORMAL_HINTS)
       (check-if-maximizable w)))
 
