@@ -102,6 +102,9 @@ typedef struct lisp_window {
     repv plist;
     repv frame_style;
 
+    /* stacking order */
+    struct lisp_window *above, *below;
+
     /* Is the client window mapped? (by its app) */
     u_int mapped : 1;
 
@@ -289,7 +292,8 @@ struct frame_part {
 
 /* codes for the clean_exit_jmp_buf */
 enum exit_codes {
-    ec_exit = 1,
+    ec_no_exit = 0,
+    ec_exit,
     ec_restart,
     ec_session_died
 };
