@@ -1,5 +1,5 @@
 /* sawmill_subrs.h -- prototypes
-   $Id: sawmill_subrs.h,v 1.92 2000/11/14 22:11:18 jsh Exp $
+   $Id: sawmill_subrs.h,v 1.93 2001/02/10 01:39:06 jsh Exp $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -343,4 +343,13 @@ extern void manage_windows (void);
 extern void windows_init (void);
 extern void windows_kill (void);
 
+/* from window-list.c */
+extern void remove_from_stacking_list (Lisp_Window *w);
+extern void insert_in_stacking_list_above_all (Lisp_Window *w);
+extern void insert_in_stacking_list_below_all (Lisp_Window *w);
+extern void insert_in_stacking_list_above (Lisp_Window *w, Lisp_Window *x);
+extern void insert_in_stacking_list_below (Lisp_Window *w, Lisp_Window *x);
+extern void restack_window (Lisp_Window *w);
+extern repv make_stacking_list (void);
+
 #endif /* SAWMILL_SUBRS_H */
